
OutputSuper = "";

-- Converts an RGB color (0 - 255) number to hex. ---------------------------------
function TO_HEX(IN)
	local B,K,OUT,I,D=16,"0123456789ABCDEF","",0
		if IN == 0 then return "00" end;
		while IN>0 do
		I=I+1
		IN,D=math.floor(IN/B),math.mod(IN,B)+1
		OUT=string.sub(K,D,D)..OUT
	end
	if string.len(OUT) == 1 then OUT = "0" .. OUT end;
	return OUT
end


-- converts the hex code to a valid Turbine color
function TOTURBINECOLOR(HEXIN)

	local sTempSource = HEXIN;
	local sLength = string.len(sTempSource);

	if (string.sub(sTempSource,1,1) ~= "#") then sTempSource = "#" .. sTempSource end

	if sLength > 7 then sTempSource = string.sub(sTempSource, 1,7) end

	sTempSource = string.upper(sTempSource);

	local iRed1 = string.sub(sTempSource,2,2);
	local iRed2 = string.sub(sTempSource,3,3);

	local iGreen1 = string.sub(sTempSource,4,4);
	local iGreen2 = string.sub(sTempSource,5,5);

	local iBlue1 = string.sub(sTempSource,6,6);
	local iBlue2 = string.sub(sTempSource,7,7);

	local sCheck = "0123456789ABCDEF";

	local iStart1,iEnd1 = string.find(sCheck,iRed1);
	local iStart2,iEnd2 = string.find(sCheck,iRed2);
	local iStart3,iEnd3 = string.find(sCheck,iGreen1);
	local iStart4,iEnd4 = string.find(sCheck,iGreen2);
	local iStart5,iEnd5 = string.find(sCheck,iBlue1);
	local iStart6,iEnd6 = string.find(sCheck,iBlue2);

	if iStart1 ~= nil then
		iStart1 = iStart1 - 1;
	else
		iStart1 = 0;
		sTempSource = string.sub(sTempSource,1,1) .. "0" .. string.sub(sTempSource,3,7);
	end;

	if iStart2 ~= nil then
		iStart2 = iStart2 - 1;
	else
		iStart2 = 0;
		sTempSource = string.sub(sTempSource,1,2) .. "0" .. string.sub(sTempSource,4,7);
	end;

	if iStart3 ~= nil then
		iStart3 = iStart3 - 1;
	else
		iStart3 = 0;
		sTempSource = string.sub(sTempSource,1,3) .. "0" .. string.sub(sTempSource,5,7);
	end;

	if iStart4 ~= nil then
		iStart4 = iStart4 - 1;
	else
		iStart4 = 0;
		sTempSource = string.sub(sTempSource,1,4) .. "0" .. string.sub(sTempSource,6,7);
	end;

	if iStart5 ~= nil then
		iStart5 = iStart5 - 1;
	else
		iStart5 = 0;
		sTempSource = string.sub(sTempSource,1,5) .. "0" .. string.sub(sTempSource,7,7);
	end;

	if iStart6 ~= nil then
		iStart6 = iStart6 - 1;
	else
		iStart6 = 0;
		sTempSource = string.sub(sTempSource,1,6) .. "0";
	end;

	local RED = (1/255)*((iStart1 * 16) + iStart2);
	local GREEN = (1/255)*((iStart3 * 16) + iStart4);
	local BLUE = (1/255)*((iStart5 * 16) + iStart6);

	local newTurbineColor = Turbine.UI.Color(RED,GREEN,BLUE);

	return newTurbineColor;

end


-- Checks the colour range is valid ------------------------------------------------

function ValColRange (colorValue)
	if (colorValue < 0) then
		return 0;
	elseif (colorValue > 1) then
		return 1;
	else
		return colorValue;
	end
end


-- Sets the color previews based on selection ---------------------------------------

function SetColor(r,g,b) -- input as Turbine colors (0 to 1)

	local tempHex = TO_HEX(math.floor(255*ValColRange(r))) .. TO_HEX(math.floor(255*ValColRange(g))) .. TO_HEX(math.floor(255*ValColRange(b)));

	lblHEX:SetText("#" .. tempHex);

	--txtOutput:SetForeColor(Turbine.UI.Color(r,g,b));

	wPreview:SetBackColor(Turbine.UI.Color(r,g,b));
	wPrevBack:SetBackColor(Turbine.UI.Color((1-r),(1-g),(1-b)));


	-- Check to see if any favourites are being replaced, if so, replace with new colour.
	for i=1, iNumFavs, 1 do

		if blFavReplace[i] == true then

			wFavColor[i]:SetBackColor(Turbine.UI.Color(r,g,b));
			wMinFavColor[i]:SetBackColor(Turbine.UI.Color(r,g,b));
			disableFavReplace(i);

		end

	end


	-- Check to see if any gradients are being replaced
	for i=1, NUMGRADIENTS, 1 do

		if blGradReplace[i] == true then

			wGradBoxCol[i]:SetBackColor(Turbine.UI.Color(r,g,b));
			disableGradReplace(i);

		end

	end


	-- Check if any text is selected, if so then apply the new colour to that part.

	if _SELECTION ~= nil then UpdateSelection(tempHex) end;

	AliasUpdate();

end


-- This function creates a table array from the string so each character can have its own formatting.
function CreateMarkupTable(STRING)

	if STRING == nil then return {} end;

	local _TEMPMARKUP = {};

	for i=1, string.len(STRING) do
		_TEMPMARKUP[i] =
			{
			["CHAR"] = string.sub(STRING,i,i);
			["COLOR"] = 0;
			};
	end

	return _TEMPMARKUP;

end


function UpdateSelection(TEMPHEX)

	if _SELECTION.START == 0 then return end;

	-- change color values in _MARKUP array to new hex values
	for i=tonumber(_SELECTION.START),tonumber(_SELECTION.END) do
		_MARKUP[i].COLOR = TEMPHEX;
	end

	--Debug(dump(_MARKUP));
	RefreshString();

	_SELECTION.START = 0;
	_SELECTION.END = 0;
	_SELECTION.TEXT = "";

end


function UpdateAll(TEMPHEX)

	-- change color values in _MARKUP array to new hex values
	local MARKUPSIZE = table.getn(_MARKUP);

	for i=1, MARKUPSIZE do
		_MARKUP[i].COLOR = TEMPHEX;
	end

	--Debug(dump(_MARKUP));
	RefreshString();

end


function RefreshString()

	-- Create output string from table..
	-- Check each color value to see if it matches the one before to cut down on <rgb> tags.
	if _MARKUP == nil then return end;


	local MARKUPSIZE = table.getn(_MARKUP);

	local tempOutputString = "";
	local curRGBTag = 0;

	for i = 1, MARKUPSIZE do

		local curCharColor = _MARKUP[i].COLOR;

		if curCharColor == curRGBTag then
			-- They are the same colours so append
			tempOutputString = tempOutputString .. _MARKUP[i].CHAR;

			if i == MARKUPSIZE and curCharColor ~= 0 then tempOutputString = tempOutputString .. "</rgb>" end;

		else
			-- They are different, check if previous RGB tag needs to be closed then create a new one
			-- in both cases, only if the color is not 0.
			if curRGBTag == 0 then
				-- Nothing to RGB
				tempOutputString = tempOutputString .. "<rgb=#" .. curCharColor .. ">" .. _MARKUP[i].CHAR;

				-- check if it's the last char.
				if i == MARKUPSIZE then tempOutputString = tempOutputString .. "</rgb>" end;

			elseif curCharColor == 0 then
				-- RGB to nothing
				tempOutputString = tempOutputString .. "</rgb>" .. _MARKUP[i].CHAR;

			else
				-- RGB to RGB
				tempOutputString = tempOutputString .. "</rgb><rgb=#" .. curCharColor .. ">" .. _MARKUP[i].CHAR;

				-- check if it's the last char.
				if i == MARKUPSIZE then tempOutputString = tempOutputString .. "</rgb>" end;

			end

		end

		curRGBTag = curCharColor;

	end


	OutputSuper = tempOutputString;

	txtOutput:SetText(OutputSuper);
	txtMinInput:SetText(OutputSuper);

	--Turbine.Shell.WriteLine(OutputSuper);

end



-- Update the alias shortcut -----------------------------------------------------
function AliasUpdate()

	local tempString = "";

	if tempChannel == "/tell" then

		local playerTell = txtTell:GetText();

		if playerTell ~= "" then

			tempString = tempChannel .. " " .. playerTell .." " .. OutputSuper;

		end

	else

		tempString = tempChannel .. " " .. OutputSuper;

	end


	qsTemp:SetShortcut(Turbine.UI.Lotro.Shortcut( Turbine.UI.Lotro.ShortcutType.Alias, tempString));
	qsMinTemp:SetShortcut(Turbine.UI.Lotro.Shortcut( Turbine.UI.Lotro.ShortcutType.Alias, tempString));

	--txtOutput:SetText(tempSource);
	--txtInput:SetText(tempString);

end


-- Update preview box as source text is edited ----------------------------------

txtInput.TextChanged = function(sender,args)

	--txtInput:SetForeColor(Turbine.UI.Color(0,0,0));

	OutputSuper = tostring(txtInput:GetText());

	_MARKUP = CreateMarkupTable(OutputSuper);

	txtOutput:SetText(OutputSuper);

	--Debug(dump(_MARKUP));

	AliasUpdate();

	if txtInput:GetTextLength() > 0 then

		btnSave:SetEnabled(true);

	else

		btnSave:SetEnabled(false);

	end

end


txtTell.FocusGained = function()

	txtTell:SetWantsUpdates(true);

end


txtTell.FocusLost = function()

	txtTell:SetWantsUpdates(false)

end


txtTell.Update = function()

	AliasUpdate();

end



tempChannel = "/say";


-- Set the channel ---------------------------------------------------------------
function SetChannel(name,command,length)
	tempChannel = command;
	lblChannel:SetText(name);
	lblMinChannel:SetText(name .. ":");
	lblMinChannel:SetWidth(length);
	txtMinInput:SetWidth(wMinParent:GetWidth() - (lblMinChannel:GetWidth() + lblMinChannel:GetLeft()) - 8)
	txtMinInput:SetLeft(lblMinChannel:GetLeft() + lblMinChannel:GetWidth() + 3);

	if name=="Tell" then
		lblTell:SetVisible(true);
		txtTell:SetVisible(true);
		wMinParent:SetVisible(false);
		wParentWin:SetVisible(true);
	else
		lblTell:SetVisible(false);
		txtTell:SetVisible(false);
		txtTell:SetText("");
	end
	AliasUpdate();
	minAliasUpdate();
end


-- Display channel menu when speech icon is clicked.------------------------------
cSpeechIcon.MouseClick = function(sender, args)
	displayMenu();
end


-- If the preview window is clicked, before SetColor function----------------------
wPreview.MouseClick = function(sender,args)

	local tmpColor = sender:GetBackColor();
	SetColor(tmpColor.R,tmpColor.G,tmpColor.B);

end


-- Send button click function -----------------------------------------------------

--[[
function btnPrev.Click(sender, args)

	tempSource = txtInput:GetText();
	tempColor = lblHEX:GetText();

	tempString = tempChannel .. " <rgb=" .. tempColor .. ">" .. tempSource .. "</rgb>";
	qsTemp:SetShortcut(Turbine.UI.Lotro.Shortcut( Turbine.UI.Lotro.ShortcutType.Alias, tempString));

	txtOutput:SetText(tempSource);
	wParentWin:Focus();

end
]]--


function btnGradient.Click(sender,args)

	drawGradWindow();
	btnGradient:SetEnabled(false);
	btnTarget:SetEnabled(false);

end


-- Handle manual hex editing ------------------------------------------------------
lblHEX.FocusLost = function()

	lblHEX:SetWantsUpdates(false);

	local sSource = lblHEX:GetText();
	local sLength = string.len(sSource);

	while sLength < 7 do
		sSource = sSource .. "0";
		sLength = sLength + 1;
	end

	lblHEX:SetText(sSource);

	local prevColor = wPreview:GetBackColor();
	local RED = prevColor.R;
	local GREEN = prevColor.G;
	local BLUE = prevColor.B;

	--SetColor(RED,GREEN,BLUE);

end


lblHEX.FocusGained = function()

	lblHEX:SetWantsUpdates(true);


end

lblHEX.Update = function()

	gblHEXCOLOR = lblHEX:GetText();

	-- Takes the value that's being entered into the HEX box and converts it into a valid Tubine color..
	-- Also deals with handling different inputs to filter out impossible color values.

	local sTempSource = lblHEX:GetText();
	local sLength = string.len(sTempSource);

	if (string.sub(sTempSource,1,1) ~= "#") then sTempSource = "#" .. sTempSource end

	if sLength > 7 then sTempSource = string.sub(sTempSource, 1,7) end

	sTempSource = string.upper(sTempSource);


	local iRed1 = string.sub(sTempSource,2,2);
	local iRed2 = string.sub(sTempSource,3,3);

	local iGreen1 = string.sub(sTempSource,4,4);
	local iGreen2 = string.sub(sTempSource,5,5);

	local iBlue1 = string.sub(sTempSource,6,6);
	local iBlue2 = string.sub(sTempSource,7,7);

	local sCheck = "0123456789ABCDEF";

	local iStart1,iEnd1 = string.find(sCheck,iRed1);
	local iStart2,iEnd2 = string.find(sCheck,iRed2);
	local iStart3,iEnd3 = string.find(sCheck,iGreen1);
	local iStart4,iEnd4 = string.find(sCheck,iGreen2);
	local iStart5,iEnd5 = string.find(sCheck,iBlue1);
	local iStart6,iEnd6 = string.find(sCheck,iBlue2);

	if iStart1 ~= nil then
		iStart1 = iStart1 - 1;
	else
		iStart1 = 0;
		sTempSource = string.sub(sTempSource,1,1) .. "0" .. string.sub(sTempSource,3,7);
	end;

	if iStart2 ~= nil then
		iStart2 = iStart2 - 1;
	else
		iStart2 = 0;
		sTempSource = string.sub(sTempSource,1,2) .. "0" .. string.sub(sTempSource,4,7);
	end;

	if iStart3 ~= nil then
		iStart3 = iStart3 - 1;
	else
		iStart3 = 0;
		sTempSource = string.sub(sTempSource,1,3) .. "0" .. string.sub(sTempSource,5,7);
	end;

	if iStart4 ~= nil then
		iStart4 = iStart4 - 1;
	else
		iStart4 = 0;
		sTempSource = string.sub(sTempSource,1,4) .. "0" .. string.sub(sTempSource,6,7);
	end;

	if iStart5 ~= nil then
		iStart5 = iStart5 - 1;
	else
		iStart5 = 0;
		sTempSource = string.sub(sTempSource,1,5) .. "0" .. string.sub(sTempSource,7,7);
	end;

	if iStart6 ~= nil then
		iStart6 = iStart6 - 1;
	else
		iStart6 = 0;
		sTempSource = string.sub(sTempSource,1,6) .. "0";
	end;

	local RED = (1/255)*((iStart1 * 16) + iStart2);
	local GREEN = (1/255)*((iStart3 * 16) + iStart4);
	local BLUE = (1/255)*((iStart5 * 16) + iStart6);


	wPreview:SetBackColor(Turbine.UI.Color(RED,GREEN,BLUE));
	wPrevBack:SetBackColor(Turbine.UI.Color((1-RED),(1-GREEN),(1-BLUE)));

	--txtOutput:SetForeColor(Turbine.UI.Color(RED,GREEN,BLUE));


	lblHEX:SetText(sTempSource);

end


-- Set up the selected favourites box for replacing ----------------------------
function enableFavReplace(sender,index)

	blFavReplace[index] = true;

	local favParent = sender:GetParent();
	favParent:SetBackColor(Turbine.UI.Color(1,1,0));
	favParent:SetSize(favParent:GetWidth()+4,favParent:GetHeight()+4);
	favParent:SetPosition(favParent:GetLeft()-2,favParent:GetTop()-2);
	sender:SetPosition(3,3);

end


-- Restore the selected favourites box after replacing --------------------------
function disableFavReplace(index)

	blFavReplace[index] = false;

	wFavBack[index]:SetBackColor(Turbine.UI.Color(1,1,1));
	wFavBack[index]:SetSize(wFavBack[index]:GetWidth()-4,wFavBack[index]:GetHeight()-4);
	wFavBack[index]:SetPosition(wFavBack[index]:GetLeft()+2,wFavBack[index]:GetTop()+2);
	wFavColor[index]:SetPosition(1,1);

end

-- Set up the selected gradient box for replacing ----------------------------
function enableGradReplace(sender,index)

	blGradReplace[index] = true;

	local gradParent = sender:GetParent();
	gradParent:SetBackColor(Turbine.UI.Color(1,1,0));
	gradParent:SetSize(gradParent:GetWidth()+2,gradParent:GetHeight()+2);
	gradParent:SetPosition(gradParent:GetLeft()-1,gradParent:GetTop()-1);
	sender:SetPosition(2,2);

end


-- Restore the selected favourites box after replacing --------------------------
function disableGradReplace(index)

	blGradReplace[index] = false;

	wGradBox[index]:SetBackColor(Turbine.UI.Color(1,1,1));
	wGradBox[index]:SetSize(wGradBox[index]:GetWidth()-2,wGradBox[index]:GetHeight()-2);
	wGradBox[index]:SetPosition(wGradBox[index]:GetLeft()+1,wGradBox[index]:GetTop()+1);
	wGradBoxCol[index]:SetPosition(1,1);

end


-- Update send button depending on mouse position --------------------------------

qsTemp.MouseEnter = function(sender,args)

	btnSend:SetBackground("GaluhadPlugins/ChatEdit/resources/send_over.jpg")

end

qsTemp.MouseLeave = function(sender,args)

	btnSend:SetBackground("GaluhadPlugins/ChatEdit/resources/send_up.jpg")

end

qsTemp.MouseDown = function(sender,args)

	btnSend:SetBackground("GaluhadPlugins/ChatEdit/resources/send_down.jpg")

end

qsTemp.MouseUp = function(sender,args)

	qsTemp:SetWantsUpdates(true);
	btnSend:SetBackground("GaluhadPlugins/ChatEdit/resources/send_over.jpg")

end


qsTemp.Update = function ()
	qsTemp:SetWantsUpdates(false);
	txtMinInput:SetText("");
	txtInput:SetText("");
	_MARKUP = nil;
	OutputSuper = "";
	minAliasUpdate();
	AliasUpdate();
end



-- Functions to save/load data ---------------------------------------------------

function saveData()

	-- Store the favourite colors in a table, then save.
	local FavColorData = {};

	for i=1,iNumFavs,1 do

		local backColor = wFavColor[i]:GetBackColor();
		FavColorData[i] =
			{
			["R"] = backColor.R;
			["G"] = backColor.G;
			["B"] = backColor.B;
			};

	end

	Turbine.PluginData.Save(Turbine.DataScope.Account, "ChatEditFavorites", FavColorData);

	-- Store the settings in a table, then save.
	local tempSettings =
		{
		["LoadIconLeft"] = bOpen:GetLeft();
		["LoadIconTop"] = bOpen:GetTop();
		["MainWinLeft"] = wParentWin:GetLeft();
		["MainWinTop"] = wParentWin:GetTop();
		["MinWinLeft"] = wMinParent:GetLeft();
		["MinWinTop"] = wMinParent:GetTop();
		["MinWinWidth"] = wMinParent:GetWidth();
		["MainWinVisible"] = wParentWin:IsVisible();
		["MinWinVisible"] = wMinParent:IsVisible();
		};

	Turbine.PluginData.Save(Turbine.DataScope.Character, "ChatEditSettings", tempSettings);

	-- Save the Alias table if it > nil
	if tAlias ~= nil then

		Turbine.PluginData.Save(Turbine.DataScope.Account, "ChatEditAliases", tAlias);

	end

end


function loadData()

	-- Load the favorite color list
	local FavColorData = {};
	FavColorData = Turbine.PluginData.Load(Turbine.DataScope.Account, "ChatEditFavorites");

	if FavColorData ~= nil then

		-- Load the colors.

		for i=1, iNumFavs, 1 do

			if FavColorData[i] ~= nil then

				local RED = FavColorData[i].R;
				local GREEN = FavColorData[i].G;
				local BLUE = FavColorData[i].B;

				wFavColor[i]:SetBackColor(Turbine.UI.Color(RED,GREEN,BLUE));

			end

		end

	end

	-- Load the settings

	local tempSettings = {};
	tempSettings = Turbine.PluginData.Load(Turbine.DataScope.Character, "ChatEditSettings")

	local ParentVisib = true;
	local MinVisib = false;

	if tempSettings ~= nil then

		-- Load the settings.
		bOpen:SetLeft(tempSettings.LoadIconLeft);
		bOpen:SetTop(tempSettings.LoadIconTop);
		wParentWin:SetLeft(tempSettings.MainWinLeft);
		wParentWin:SetTop(tempSettings.MainWinTop);
		gblMINWINLEFT = tempSettings.MinWinLeft;
		gblMINWINTOP = tempSettings.MinWinTop;
		gblMINWINWIDTH = tempSettings.MinWinWidth;
		ParentVisib = tempSettings.MainWinVisible;
		MinVisib = tempSettings.MinWinVisible;

	end

	wAliasParent:SetPosition(bOpen:GetLeft()+23,bOpen:GetTop()-1);

	drawMinWindow();

	if ParentVisib ~= nil then wParentWin:SetVisible(ParentVisib) end;
	if MinVisib ~= nil then wMinParent:SetVisible(MinVisib) end;


	-- Load saved aliases
	tAlias = {};
	tAlias = Turbine.PluginData.Load(Turbine.DataScope.Account, "ChatEditAliases")

	if tAlias == nil then tAlias = {}; end

	CreateAliasWin();

end



Turbine.Plugin.Unload = function (sender, args)

	saveData();

end

Turbine.Plugin.Load = function (sender,args)

	loadData();

end


-- Mini-icon mouse handles ----------------------------------------------------------------
blOpenDragging = false;

bOpen.MouseEnter = function(Sender, args)

	wAliasParent:SetVisible(true);

end


bOpen.MouseLeave = function(Sender, args)

	wAliasParent:SetVisible(false);

end


bOpen.MouseDoubleClick = function (sender, args)

	if wParentWin:IsVisible() == false then

		if wMinParent:IsVisible() == false then wParentWin:SetVisible(true) end;

	end

end


bOpen.MouseDown = function (sender, args)

	blOpenDragging = true;

end


bOpen.MouseUp = function (sender, args)

	blOpenDragging = false;

end


bOpen.MouseMove = function (sender, args)

	if blOpenDragging == true then

		local curX = Turbine.UI.Display.GetMouseX();
		local curY = Turbine.UI.Display.GetMouseY();
		local scWidth = Turbine.UI.Display.GetWidth();
		local scHeight = Turbine.UI.Display.GetHeight();

		local curLeft = curX - 12;
		local curTop = curY -12;

		if (curX-12) < 0 then curLeft = 0 end;
		if (curX+12) > scWidth then curLeft = scWidth-23 end;
		if (curY-12) < 0 then curTop = 0 end;
		if (curY+12) > scHeight then curTop = scHeight-23 end;

		bOpen:SetLeft(curLeft);
		bOpen:SetTop(curTop);

		wAliasParent:SetTop(curTop-1);

		local farRight = bOpen:GetLeft() + bOpen:GetWidth() + wAliasParent:GetWidth();

		if farRight > Turbine.UI.Display.GetWidth() then

			wAliasParent:SetLeft(bOpen:GetLeft()-wAliasParent:GetWidth());

		else

			wAliasParent:SetLeft(bOpen:GetLeft()+bOpen:GetWidth());

		end

	end

end

blGradDragging = false;

-- Function to add new colour boxes to the gradient slider.
function gradAdd(mouseX)

	newLeft = mouseX-12;

	-- Need to create a buffer table to find the positions of the other color blocks, and then to
	-- be able to insert the new color block at the right location in the table so everything
	-- remains in order.
	local initTableBuffer = {};

	for i=1, NUMGRADIENTS, 1 do

		initTableBuffer[i] =
			{
			["parent"] = wGradBox[i];
			["child"] = wGradBoxCol[i];
			};

	end

	-- Increase the global number of gradient colours.
	NUMGRADIENTS = NUMGRADIENTS + 1;

	-- Check the new index location based on where the mouse was clicked in relation to other blocks.
	local tempBlockPosition = 0;
	local index = 0;

	while tempBlockPosition < newLeft do

		index = index + 1;
		tempBlockPosition = initTableBuffer[index].parent:GetLeft();

	end


	-- A new table buffer to store the leading indexes, the new index, and the following indexes.
	updatedTableBuffer = {};

	-- Leading indexes
	for i=1, (index-1), 1 do

		updatedTableBuffer[i] =
			{
			["parent"] = initTableBuffer[i].parent;
			["child"] = initTableBuffer[i].child;
			};

	end

	-- New index to insert.
	updatedTableBuffer[index] =
		{
		["parent"] = Turbine.UI.Window();
		["child"] = Turbine.UI.Window();
		};

	updatedTableBuffer[index].parent:SetParent(wGradBack);
	updatedTableBuffer[index].parent:SetSize(24,24);
	updatedTableBuffer[index].parent:SetPosition(newLeft,0);
	updatedTableBuffer[index].parent:SetBackColor(Turbine.UI.Color(1,1,1));
	updatedTableBuffer[index].parent:SetVisible(true);

	updatedTableBuffer[index].child:SetParent(updatedTableBuffer[index].parent);
	updatedTableBuffer[index].child:SetSize(22,22);
	updatedTableBuffer[index].child:SetPosition(1,1);
	updatedTableBuffer[index].child:SetBackColor(Turbine.UI.Color(0.2,0.2,0.2));
	updatedTableBuffer[index].child:SetVisible(true);


	-- Following indexes to add after the new block
	for i=index+1, NUMGRADIENTS, 1 do

		updatedTableBuffer[i] =
			{
			["parent"] = initTableBuffer[(i-1)].parent;
			["child"] = initTableBuffer[(i-1)].child;
			};

	end


	-- Now we have the new table buffer complete, copy it back to the screen
	for i=1, NUMGRADIENTS, 1 do

		wGradBox[i] = updatedTableBuffer[i].parent;
		wGradBoxCol[i] = updatedTableBuffer[i].child;

		wGradBoxCol[i].MouseClick = function (sender,args)

			if (args.Button == Turbine.UI.MouseButton.Right) then

				displayGradMenu(sender,i);

			else

				colorString = sender:GetBackColor();
				SetColor(colorString.R,colorString.G,colorString.B);

			end

		end

		wGradBoxCol[i].MouseDown = function (sender, args)

			if i > 1 and i < NUMGRADIENTS then
				blGradDragging = true;
			end

		end

		wGradBoxCol[i].MouseUp = function (sender, args)

			blGradDragging = false;

		end

		wGradBoxCol[i].MouseMove = function (sender, args)

			if blGradDragging == true then

				local newX = Turbine.UI.Display.GetMouseX();
				local newLeft = wGradBack:PointToClient(newX - 12,0);
				local minLeft = wGradBox[(i-1)]:GetLeft() + wGradBox[(i-1)]:GetWidth();
				local maxLeft = wGradBox[i+1]:GetLeft() - 24;

				if newLeft < minLeft then newLeft = minLeft end;
				if newLeft > maxLeft then newLeft = maxLeft end;

				wGradBox[i]:SetLeft(newLeft);

			end

		end

	end

end


function gradRemove (index)

	NUMGRADIENTS = NUMGRADIENTS - 1; -- Global gradient variable

	-- Remove visibility of the selected window.
	wGradBox[index]:SetVisible(false);

	-- Create a new table buffer which will combine the leading blocks and following blocks
	-- but leave out the selected block/index.
	local TableBuffer = {};

	-- Leading blocks
	for i=1, (index-1), 1 do

		TableBuffer[i] =
			{
			["parent"] = wGradBox[i];
			["child"] = wGradBoxCol[i];
			};

	end

	--Following blocks
	for i=index, NUMGRADIENTS, 1 do

		TableBuffer[i] =
			{
			["parent"] = wGradBox[(i+1)];
			["child"] = wGradBoxCol[(i+1)];
			};

	end

	-- Now we have the new table buffer complete, copy it back to the screen
	for i=1, NUMGRADIENTS, 1 do

		wGradBox[i] = TableBuffer[i].parent;
		wGradBoxCol[i] = TableBuffer[i].child;

		wGradBoxCol[i].MouseClick = function (sender,args)

			if (args.Button == Turbine.UI.MouseButton.Right) then

				displayGradMenu(sender,i);

			else

				colorString = sender:GetBackColor();
				SetColor(colorString.R,colorString.G,colorString.B);

			end

		end

		wGradBoxCol[i].MouseDown = function (sender, args)

			if i > 1 and i < NUMGRADIENTS then
				blGradDragging = true;
			end

		end

		wGradBoxCol[i].MouseUp = function (sender, args)

			blGradDragging = false;

		end

		wGradBoxCol[i].MouseMove = function (sender, args)

			if blGradDragging == true then

				local newX = Turbine.UI.Display.GetMouseX();
				local newLeft = wGradBack:PointToClient(newX - 12,0);
				local minLeft = wGradBox[(i-1)]:GetLeft() + wGradBox[(i-1)]:GetWidth();
				local maxLeft = wGradBox[i+1]:GetLeft() - 24;

				if newLeft < minLeft then newLeft = minLeft end;
				if newLeft > maxLeft then newLeft = maxLeft end;

				wGradBox[i]:SetLeft(newLeft);

			end

		end

	end

end


wAliasParent.MouseEnter = function (sender,args)

		wAliasParent:SetVisible(true);

end

wAliasParent.MouseLeave = function (sender,args)

		wAliasParent:SetVisible(false);

end


btnSave.Click = function (sender,args)

	saveAlias();

end

-- Min button ---------------------------------------------------------------

btnMin.MouseEnter = function(sender,args)

	btnMin:SetBackground("GaluhadPlugins/ChatEdit/resources/min_over.jpg")

end

btnMin.MouseLeave = function(sender,args)

	btnMin:SetBackground("GaluhadPlugins/ChatEdit/resources/min_up.jpg")

end

btnMin.MouseDown = function(sender,args)

	btnMin:SetBackground("GaluhadPlugins/ChatEdit/resources/min_down.jpg")

end

btnMin.MouseUp = function(sender,args)

	btnMin:SetBackground("GaluhadPlugins/ChatEdit/resources/min_over.jpg")
	txtMinInput:SetForeColor(Turbine.UI.Color(1,1,1));
	txtMinInput:SetText(OutputSuper);
	gblHEXCOLOR = string.sub(lblHEX:GetText(), 2);
	minAliasUpdate();

	if lblChannel:GetText() == "Tell" then
		lblMinChannel:SetText("Say:");
		tempChannel = "/say";
		minAliasUpdate();
	end;

	wMinParent:SetVisible(true);
	wParentWin:SetVisible(false);

end


btnTarget.MouseClick = function (sender, args)

	-- Adds ;target to the text
	txtInput:InsertText(";target");

	OutputSuper = tostring(txtInput:GetText());

	_MARKUP = CreateMarkupTable(OutputSuper);

	RefreshString();
	AliasUpdate();

end
